@@ -2,6 +2,7 @@
 #define SPEAK_EASY_H
 
 #include <igraph.h>
+#include "../src/speakeasy2/se2_interface.h"
 
 typedef struct {
   igraph_integer_t independent_runs;  // Number of independent runs to perform.
@@ -20,10 +21,22 @@ typedef struct {
   bool verbose; // Print information to stdout
 } se2_options;
 
-igraph_error_t speak_easy_2(igraph_t* graph, igraph_vector_t* weights,
-                            se2_options* opts, igraph_matrix_int_t* res);
-igraph_error_t se2_order_nodes(igraph_t const* graph,
-                               igraph_vector_t const* weights,
+typedef struct {
+  igraph_vector_int_list_t* neigh_list;
+  igraph_vector_list_t* weights;
+  igraph_vector_int_t* sizes;
+  igraph_integer_t n_nodes;
+  igraph_vector_t* kin;
+  igraph_real_t total_weight;
+} se2_neighs;
+
+igraph_error_t se2_igraph_to_neighbor_list(igraph_t const* graph,
+    igraph_vector_t const* weights, se2_neighs* neigh_list);
+void se2_neighs_destroy(se2_neighs* graph);
+
+igraph_error_t speak_easy_2(se2_neighs* graph, se2_options* opts,
+                            igraph_matrix_int_t* res);
+igraph_error_t se2_order_nodes(se2_neighs const* graph,
                                igraph_matrix_int_t const* memb,
                                igraph_matrix_int_t* ordering);
 igraph_error_t se2_knn_graph(igraph_matrix_t* mat, igraph_integer_t const k,
